function [fh, pyr, pind] = decomp_reconst_full_video(Nsc,Nor,block,noise,parent,covariance,optim,sig, numFrames, frmDepth, pyr, pind);

% Decompose video into subbands, denoise, and recompose again.
%		fh = decomp_reconst_full_video(Nsc,Nor,block,noise,parent,covariance,optim,sig, numFrames, frmDepth, pyr, pind);
%       covariance:	are we considering covariance or just variance?
%       optim     :	for choosing between BLS-GSM (optim = 1) and MAP-GSM (optim = 0)
%       sig       : standard deviation (scalar for uniform noise or matrix for spatially varying noise)
%       numframes : number of frames used for denoising
%       frmDepth  : number of frames required in forward/backward direction
%       pyr       : pyramidal decomposition of past and future frames
%       pind      : pyramid indices 
% Version using the Full steerable pyramid (2) (High pass residual
% splitted into orientations).
% Original version: Javier Portilla, Univ. de Granada, 11/04 (BLS-Image denoising)
% Modified revision: 02/07.Gijesh Varghese, 
           


if (block(1)/2==floor(block(1)/2))|(block(2)/2==floor(block(2)/2)),
   error('Spatial dimensions of neighborhood must be odd!');
end   

if ~exist('parent'),
        parent = 1;
end

if ~exist('covariance'),
        covariance = 1;
end

if ~exist('optim'),
        optim = 1;
end

if ~exist('sig'),
        sig = sqrt(mean(noise.^2));
end

[pyrN,pindn] = buildFullSFpyr2(noise,Nsc,Nor-1);
pyrh = real(pyr(:,1));
Nband = size(pind(:,:,1),1)-1;
for nband = 2:Nband, % everything except the low-pass residual
  fprintf('%d % ',round(100*(nband-1)/(Nband-1)))
  aux = pyrBand(pyr(:,1), pind(:,:,1), nband);
  auxn = pyrBand(pyrN, pindn, nband);
  [Nsy,Nsx] = size(aux);
  prnt = parent & (nband < Nband-Nor);   % has the subband a parent?
  BL = zeros(size(aux,1),size(aux,2), prnt+numFrames);
  BLn = zeros(size(aux,1), size(aux,2),prnt+1);
  BL(:,:,1) = aux;
  BLn(:,:,1) = auxn*sqrt(((Nsy-2)*(Nsx-2))/(Nsy*Nsx));     % because we are discarding 2 coefficients on every dimension  
  
  %stacking coeffcients of adjacent frames
  for k=1:numFrames-1
      BL(:,:,1+prnt+k) = pyrBand( pyr(:,k+1), pind(:,:,k+1), nband);
  end

  if prnt,
  	aux = pyrBand(pyr(:,1), pind(:,:,1), nband+Nor);
    auxn = pyrBand(pyrN, pindn, nband+Nor);
    if nband>Nor+1,     % resample 2x2 the parent if not in the high-pass oriented subbands.
	   aux = real(expand(aux,2));
       auxn = real(expand(auxn,2));
    end    
  	BL(:,:,2) = aux;
    BLn(:,:,2) = auxn*sqrt(((Nsy-2)*(Nsx-2))/(Nsy*Nsx)); % because we are discarding 2 coefficients on every dimension       
  end
   
  sy2 = mean2(BL(:,:,1).^2);
  sn2 = mean2(BLn(:,:,1).^2);
  if sy2>sn2,
     SNRin = 10*log10((sy2-sn2)/sn2);
  else
     disp('Signal is not detectable in noisy subband');
  end   
  
  % main
  BL = denoi_BLS_GSM_band_video(BL,block,BLn,prnt,covariance,optim,sig,numFrames);
   pyrh(pyrBandIndices(pind(:,:,1),nband)) = BL(:)';
end
fh = reconFullSFpyr2(pyrh,pind(:,:,1));
pyr = pyr;
pind=pind;